@@ -1,10 +1,11 @@
 package Neovim::Ext;
-$Neovim::Ext::VERSION = '0.04';
+$Neovim::Ext::VERSION = '0.05';
 use strict;
 use warnings;
 use base qw/Class::Accessor/;
 use Carp;
 use Exporter qw/import/;
+use Scalar::Util qw/blessed/;
 use Neovim::Ext::MsgPack::RPC;
 use Neovim::Ext::Common qw/walk/;
 use Neovim::Ext::Buffer;
@@ -60,10 +61,16 @@ sub _setup_logging
 		tie (*STDERR => 'Neovim::Ext::Tie::Stream', sub
 			{
 				my ($data) = @_;
-				$data .= "\n" if (index ($data, "\n") == -1);
-				$nvim->err_write ($data, async_ => 1);
 
-				print $stderr $data if ($stderr);
+				if ($stderr)
+				{
+					print $stderr $data if ($stderr);
+				}
+				else
+				{
+					$data .= "\n" if (substr ($data, -1) ne "\n");
+					$nvim->err_write ($data, async_ => 1);
+				}
 			}
 		);
 
@@ -74,7 +81,7 @@ sub _setup_logging
 		tie (*NEWSTDOUT => 'Neovim::Ext::Tie::Stream', sub
 			{
 				my ($data) = @_;
-				$data .= "\n" if (index ($data, "\n") == -1);
+				$data .= "\n" if (substr ($data, -1) ne "\n");
 				$nvim->out_write ($data, async_ => 1);
 			}
 		);
@@ -175,7 +182,16 @@ sub run_loop
 
 	$err_cb //= sub
 	{
-		print STDERR @_;
+		my ($exception) = @_;
+
+		if (ref ($exception) && ref ($exception) eq 'Neovim::Ext::ErrorResponse')
+		{
+			print STDERR $exception->{msg};
+		}
+		else
+		{
+			print STDERR $exception;
+		}
 	};
 
 	$this->err_cb ($err_cb);
@@ -432,7 +448,7 @@ sub _to_nvim
 {
 	my ($this, $obj) = @_;
 
-	if (ref ($obj) && $obj->isa ('Neovim::Ext::Remote'))
+	if (ref ($obj) && blessed ($obj) && $obj->isa ('Neovim::Ext::Remote'))
 	{
 		return $obj->code_data;
 	}
@@ -477,7 +493,7 @@ Neovim::Ext - Perl bindings for neovim
 
 =head1 VERSION
 
-version 0.04
+version 0.05
 
 =head1 DESCRIPTION
 
