@@ -1,5 +1,5 @@
 package Neovim::Ext::Buffer;
-$Neovim::Ext::Buffer::VERSION = '0.04';
+$Neovim::Ext::Buffer::VERSION = '0.05';
 use strict;
 use warnings;
 use Tie::Array;
@@ -48,7 +48,18 @@ sub FETCH
 sub STORE
 {
 	my ($this, $index, $value) = @_;
-	$this->request ('nvim_buf_set_lines', $index, $index+1, 0, defined ($value) ? [$value] : []);
+	$this->request ('nvim_buf_set_lines', $index, $index+1, 0, defined ($value) ? ["$value"] : []);
+}
+
+
+
+sub SPLICE
+{
+	my ($this, $offset, $length, @list) = @_;
+
+	$offset //= 0;
+	$length //= $this->request ('nvim_buf_line_count');
+	$this->request ('nvim_buf_set_lines', $offset, $offset+$length, 0, [map { "$_" } @list]);
 }
 
 
@@ -134,7 +145,7 @@ Neovim::Ext::Buffer - Neovim Buffer class
 
 =head1 VERSION
 
-version 0.04
+version 0.05
 
 =head1 SYNPOSIS
 
