@@ -1,5 +1,5 @@
 package Neovim::Ext::Plugin::ScriptHost;
-$Neovim::Ext::Plugin::ScriptHost::VERSION = '0.04';
+$Neovim::Ext::Plugin::ScriptHost::VERSION = '0.05';
 use strict;
 use warnings;
 use List::Util qw/min/;
@@ -7,8 +7,9 @@ use base 'Neovim::Ext::Plugin';
 use Neovim::Ext::ErrorResponse;
 use Neovim::Ext::VIMCompat::Buffer;
 use Neovim::Ext::VIMCompat::Window;
+use Eval::Safe;
 
-__PACKAGE__->mk_accessors (qw/current/);
+__PACKAGE__->mk_accessors (qw/current env/);
 __PACKAGE__->register;
 
 BEGIN
@@ -17,6 +18,13 @@ BEGIN
 };
 
 our $VIM;
+our $curbuf;
+our $curwin;
+our $line;
+our $linenr;
+our $current;
+our $vim;
+our $nvim;
 
 
 sub new
@@ -25,7 +33,17 @@ sub new
 
 	$VIM = $nvim;
 
-	return $this->SUPER::new ($nvim, $host);
+	my $obj = $this->SUPER::new ($nvim, $host);
+	$obj->env (Eval::Safe->new());
+	$obj->env->share ('$curbuf');
+	$obj->env->share ('$curwin');
+	$obj->env->share ('$line');
+	$obj->env->share ('$linenr');
+	$obj->env->share ('$current');
+	$obj->env->share ('$vim');
+	$obj->env->share ('$nvim');
+
+	return $obj;
 }
 
 sub perl_execute :nvim_rpc_export('perl_execute', sync => 1)
@@ -64,25 +82,24 @@ sub perl_do_range :nvim_rpc_export('perl_do_range', sync => 1)
 
 	$start -= 1;
 
+	my $buffer = tied (@{$this->nvim->current->buffer});
 	while ($start < $stop)
 	{
-		my $sstart = $start;
-		my $sstop = min ($start + 5000, $stop);
-		my $lines = tied (@{$this->nvim->current->buffer})->api->get_lines ($sstart, $sstop, 1);
+		my $lines = $buffer->api->get_lines ($start, $start+1, 1);
 
-		my @newlines;
-		my $linenr = $sstart + 1;
-		foreach my $line (@$lines)
-		{
-			my $result = $this->_eval ($start, $stop, $code, $line, $linenr);
-			push @newlines, $result;
-			++$linenr;
-		}
+		$this->_setup_current ($start, $start, $lines->[0], $start+1);
+		my $result = $this->env->eval ("local \$_ = \$line; $code; \$_") // '';
 
-		$start = $sstop;
+		$buffer->api->set_lines ($start, $start+1, 1, [$result]);
 
-		tied (@{$this->nvim->current->buffer})->api->set_lines ($sstart, $sstop, 1, \@newlines);
+		++$start;
+
+		# The number of lines in the buffer could have reduced and $stop
+		# could now point past the end, readjust.
+		my $count = scalar (@{$this->nvim->current->buffer});
+		$stop = min ($count, $stop);
 	}
+
 	return undef;
 }
 
@@ -90,10 +107,8 @@ sub perl_eval :nvim_rpc_export('perl_eval', sync => 1)
 {
 	my ($this, $expr) = @_;
 
-	# Bringe $current, $vim and $nvim into lexical scope
-	my ($vim, $nvim) = ($this->nvim, $this->nvim);
-
-	return eval $expr;
+	$this->_setup_current();
+	return $this->env->eval ($expr) // 0;
 }
 
 sub perl_chdir :nvim_rpc_export('perl_chdir', sync => 0)
@@ -104,41 +119,28 @@ sub perl_chdir :nvim_rpc_export('perl_chdir', sync => 0)
 
 sub _eval
 {
-	my ($this, $start, $stop, $code, $line, $linenr) = @_;
+	my ($this, $start, $stop, $code) = @_;
 
-	# Bringe $vim and $nvim into lexical scope
-	my $current = $this->nvim->current;
-	my ($vim, $nvim) = ($this->nvim, $this->nvim);
+	$this->_setup_current ($start, $stop);
+	$this->env->eval ($code);
+}
 
-	$current->range (tied (@{$current->buffer})->range ($start, $stop));
+sub _setup_current
+{
+	my ($this, $start, $stop, $line_, $linenr_) = @_;
 
-	my $curbuf = Neovim::Ext::VIMCompat::Buffer->new ($current->buffer);
-	my $curwin = Neovim::Ext::VIMCompat::Window->new ($current->window);
+	$vim = $this->nvim;
+	$nvim = $this->nvim;
 
+	$current = $this->nvim->current;
+	$current->range (tied (@{$current->buffer})->range ($start, $stop)) if (defined ($start) && defined ($stop));
+	$curbuf = Neovim::Ext::VIMCompat::Buffer->new ($current->buffer);
+	$curwin = Neovim::Ext::VIMCompat::Window->new ($current->window);
 	$main::curbuf = $curbuf;
 	$main::curwin = $curwin;
 
-	local $_ = $line if ($line);
-
-	my $script =
-		"package main;\n".
-		"no strict;\n".
-		"no warnings;\n";
-
-	if (defined ($line) && defined ($linenr))
-	{
-		$script .=
-			"my \$line = \"$line\";\n".
-			"my \$linenr = $linenr;\n".
-			"\$_ = \$line;\n";
-	}
-
-	$script .=
-		"$code;\n".
-		"1;\n";
-
-	eval $script;
-	return $_;
+	$line = $line_;
+	$linenr = $linenr_;
 }
 
 =head1 NAME
@@ -147,7 +149,7 @@ Neovim::Ext::Plugin::ScriptHost - Neovim Legacy perl Plugin
 
 =head1 VERSION
 
-version 0.04
+version 0.05
 
 =head1 SYNOPSIS
 
