@@ -1,5 +1,5 @@
 package Neovim::Ext::Range;
-$Neovim::Ext::Range::VERSION = '0.04';
+$Neovim::Ext::Range::VERSION = '0.05';
 use strict;
 use warnings;
 use base qw/Class::Accessor Tie::Array/;
@@ -83,11 +83,19 @@ sub _normalise_index
 
 =head1 NAME
 
-A remote Nvim buffer range. A C<Neovim::Ext::Range> instance is a tied array reference.
+Neovim::Ext::Range - Neovim Buffer range class
 
 =head1 VERSION
 
-version 0.04
+version 0.05
+
+=head1 SYNPOSIS
+
+	use Neovim::Ext;
+
+=head1 DESCRIPTION
+
+A buffer range for a C<Neovim::Ext::Buffer>.
 
 =cut
 
