@@ -1,5 +1,5 @@
 package Neovim::Ext::VIMCompat::Window;
-$Neovim::Ext::VIMCompat::Window::VERSION = '0.04';
+$Neovim::Ext::VIMCompat::Window::VERSION = '0.05';
 use strict;
 use warnings;
 use base qw/Class::Accessor/;
@@ -42,7 +42,13 @@ sub Buffer
 sub Cursor
 {
 	my ($this, $row, $col) = @_;
-	$this->window->cursor ($row, $col);
+
+	if (defined ($row) || defined ($col))
+	{
+		$this->window->cursor ([$row, $col]);
+	}
+
+	return @{$this->window->cursor};
 }
 
 =head1 NAME
@@ -51,7 +57,7 @@ Neovim::Ext::VIMCompat::Windo - Neovim legacy VIM perl compatibility layer
 
 =head1 VERSION
 
-version 0.04
+version 0.05
 
 =head1 SYNPOSIS
 
